/*

POIs import

POI import needs the following templated fields :
poi_type: integer, type of the POI (1 - car park, 4 - shared cycle point, etc.), default to 5 (user POI)
name : string, field containing the name of each POI, default to 'pname'
service_name : string, name of the service (Bicloo, Velov, Marguerite)
filter: string, WHERE clause of the import, default to 'true' (no filter)

*/

BEGIN;

INSERT INTO tempus.transport_mode (id, name,public_transport,traffic_rules,speed_rule,toll_rule,shared_vehicle,return_shared_vehicle,need_parking)
   SELECT
   (SELECT max(id)+1 FROM tempus.transport_mode), 
   CASE WHEN %(poi_type) = 2 THEN
        'Shared car (%(service_name))'
        WHEN %(poi_type) = 4 THEN
        'Shared bike (%(service_name))'
   END,
   'f', -- public transport
   CASE WHEN %(poi_type) = 2 THEN 4 ELSE 2 END, -- traffic_rules
   CASE WHEN %(poi_type) = 2 THEN 4 ELSE 2 END, -- speed_rule
   CASE WHEN %(poi_type) = 2 THEN 4 ELSE 2 END, -- toll_rule
   't', -- shared_vehicule
   't', -- return_shared_vehicle
   't' -- need_parking
   WHERE %(poi_type) in (2,4);

drop sequence if exists _tempus_import.poi_id;
create sequence _tempus_import.poi_id start with 1;
select setval('_tempus_import.poi_id', (select case when max(id) is null then 1 else max(id)+1 end from tempus.poi), false);

do $$
declare
  poi record;
begin
  for poi in select * from _tempus_import.poi where %(filter) loop
    insert into tempus.poi (
            id
            , poi_type
            , name
            , parking_transport_modes
            , road_section_id
            , abscissa_road_section
            , geom
        )
    select
      nextval('_tempus_import.poi_id')::bigint as id
      , %(poi_type)::integer as poi_type
      , poi.%(name) as name
      , array[(select
      case
          when %(poi_type) = 1 -- car park
               then 3 -- private car
          when %(poi_type) = 2 -- shared car
               then (select max(id) from tempus.transport_mode) -- last inserted transport mode
          when %(poi_type) = 3 -- bike
               then 2 -- private bike
          when %(poi_type) = 4 -- shared bike
               then (select max(id) from tempus.transport_mode) -- last inserted transport mode
          else
               null
      end)]
      , road_section_id
      , st_LineLocatePoint(geom_road, st_transform(poi.geom, 4326))::double precision as abscissa_road_section
      , st_Force3DZ(st_transform(poi.geom, 4326))
    from
      (
        select
          rs.id as road_section_id
          , rs.geom as geom_road
        from
          tempus.road_section rs
        order by
          st_transform(poi.geom, 4326) <-> rs.geom
        limit 10
      ) t
    order by st_distance(geom_road, st_transform(poi.geom, 4326))
    limit 1;
  end loop;
end$$;
    
COMMIT;
